# -*- coding: utf-8 -*-
"""
Created on Feb 26 2018

@author: Domen
@source:  https://damogranlabs.com/2018/03/rc-filter-frequency-pole-calculator/
"""

import sys
import math

########### HIGH PASS FILTER - differentiator
LOW_FREQ = 20  # [Hz] lower frequency  (HIGH PASS FILTER)
LOW_FREQ_DEV = 0.5  # [dB] response deviation at LOW frequency

# set either XP_RESISTOR_CALCULATE_CAPACITOR or XP_CAPACITOR_CALCULATE_RESISTOR or none of them
HP_RESISTOR_CALCULATE_CAPACITOR = True
RESISTOR_HP = 10e3    #[ohm]
HP_CAPACITOR_CALCULATE_RESISTOR = False
CAPACITOR_HP = 1e-6   #[F]


########### LOW PASS FILTER - integrator
HIGH_FREQ = 20e3  # [Hz] upper frequency (LOW PASS FILTER)
HIGH_FREQ_DEV = 0.5  # [dB] response deviation at HIGH frequency

# set either XP_RESISTOR_CALCULATE_CAPACITOR or XP_CAPACITOR_CALCULATE_RESISTOR or none of them
LP_RESISTOR_CALCULATE_CAPACITOR = True
RESISTOR_LP = 10e3    # [ohm]
LP_CAPACITOR_CALCULATE_RESISTOR = False
CAPACITOR_LP = 2.2e-6   #[F]

DETAIL = True

#########################################################################################
# private
#########################################################################################
LOW_PASS = 0
HIGH_PASS = 1

if (LP_RESISTOR_CALCULATE_CAPACITOR and LP_CAPACITOR_CALCULATE_RESISTOR) or \
    (HP_RESISTOR_CALCULATE_CAPACITOR and HP_CAPACITOR_CALCULATE_RESISTOR):
    print("\t**** ERROR in freq_calc.py ****")
    print("Both XP_RESISTOR_CALCULATE_CAPACITOR and XP_CAPACITOR_CALCULATE_RESISTOR are set to 'True'!")
    print("Define only one (or none) of them for each filter (LP or HP).\n")
    sys.exit(1)

def detail(msg):
    """
    Print message if DETAIL is True.
    """
    if DETAIL:
        msg = str(msg)
        print("Detail: " + msg)


def get_pole_freq(freq, dev, lp_hp):
    """
    Get frequency pole (-3dB point) of filter, which has '-dev' dB deviation ad 'freq'.
    
    freq - wanted frequency pole [Hz]
    dev - deviation at wanted frequency ('freq') pole [dB]
    lp_hp - LOW_PASS or HIGH_PASS filter

    Returns frequency pole (float) [Hz].
    """

    """
    G0 - reference gain = 1
    Gf - gain at freq
    Converts 'dev' from dB to gain.
    """
    _g_0 = 1  # reference gain
    _g_f = math.pow(10, -dev / 20) * _g_0  # relative gain
    detail("Response deviation of %.2f dB => relative gain of %.4f to reference %.2f" %
        (-dev, _g_f, _g_0))

    """
    A response deviation of -dev at freq is acceptable. The location of
    the pole can be calculated using the relative gain at freq:
    """
    
    if lp_hp == LOW_PASS:
        _transfer_f = _g_0 / _g_f
        _transfer_f_ = math.pow(_transfer_f, 2) - 1
        freq_p = freq / math.sqrt(_transfer_f_)
    
    else:   # high pass filter
        _transfer_f = _g_0 / _g_f
        _transfer_f_ = math.pow(_transfer_f, 2) - 1
        freq_p = freq * math.sqrt(_transfer_f_)

    detail("Pole frequency is %.2f Hz with %.2f dB deviation at %d Hz)" %
        (freq_p, -dev, freq))
    return freq_p

def get_res_cap_value(freq, res_cap):
    """
    Get resistor/capacitor value for chosen frequency pole and resistor/capacitor.
    
    freq - calculated frequency pole (-3dB point)
    res_cap - chosen capacitor value in Farads or resistor in ohms
    
    Returns resistor/capacitor value in ohms/Farads.
    """
    return (1 / (2 * math.pi * freq * res_cap))


#########################################################################################
def main():
    if DETAIL:
        print()

    # HIGH PASS FILTER (takes low frequency pole)
    f_hp = get_pole_freq(LOW_FREQ, LOW_FREQ_DEV, HIGH_PASS)
    print("\nHigh pass frequency pole is at %.2f Hz for -%.2f dB deviation at %d Hz." %(f_hp, LOW_FREQ_DEV, LOW_FREQ))

    if HP_CAPACITOR_CALCULATE_RESISTOR:
        res_hp = get_res_cap_value(f_hp, CAPACITOR_HP)
        print("\tRC values:\n\t\tR: %.2e ohm (calculated)\n\t\tC: %.2e F" %(res_hp, CAPACITOR_HP))

    elif HP_RESISTOR_CALCULATE_CAPACITOR:
        cap_hp = get_res_cap_value(f_hp, RESISTOR_HP)
        print("\tRC values:\n\t\tR: %.2e ohm\n\t\tC: %.2e F (calculated)" %(RESISTOR_HP, cap_hp))

    # LOW PASS FILTER (takes high frequency pole)
    f_lp = get_pole_freq(HIGH_FREQ, HIGH_FREQ_DEV, LOW_PASS)
    print("\nLow pass frequency pole is at %.2f Hz for -%.2f dB deviation at %d Hz." %(f_lp, HIGH_FREQ_DEV, HIGH_FREQ))

    if LP_CAPACITOR_CALCULATE_RESISTOR:
        res_lp = get_res_cap_value(f_lp, CAPACITOR_LP)
        print("\tRC values:\n\t\tR: %.2e ohm (calculated)\n\t\tC: %.2e F" %(res_lp, CAPACITOR_LP))

    elif LP_RESISTOR_CALCULATE_CAPACITOR:
        cap_lp = get_res_cap_value(f_lp, RESISTOR_LP)
        print("\tRC values:\n\t\tR: %.2e ohm\n\t\tC: %.2e F (calculated)" %(RESISTOR_LP, cap_lp))

    print()


if __name__ == "__main__":
    # execute only if run as a script
    main()